(function () {
  function latitudeToMercator(latitude) {
    return Math.log(Math.tan(latitude * Math.PI / 180) + 1 / Math.cos(latitude * Math.PI / 180));
  }

  function getLocation() {
    if ($('#edit-html5-user-geolocation-save').attr('checked')) {
      var $map = $('#html5-user-geolocation-map'),
        $busy = $('#html5-user-geolocation-messages-wrapper .geolocating');
      $('#html5-user-geolocation-map-wrapper').slideDown('fast');

      // Get position
      $busy.show();
      navigator.geolocation.getCurrentPosition(function (position) {
        // Plot coords
        $('#html5-user-geolocation-map .dot')
        .css('left', '' + ((position.coords.longitude + 168 + 360) / 360 % 1 * $map.width()) + 'px')
        .css('bottom', '' + ((latitudeToMercator(position.coords.latitude) - latitudeToMercator(-58)) / (latitudeToMercator(78) - latitudeToMercator(-58)) * $map.height()) + 'px')
        .show();

        // Save coords
        $('#edit-html5-user-geolocation-latitude').attr('value', position.coords.latitude);
        $('#edit-html5-user-geolocation-longitude').attr('value', position.coords.longitude);

        // Show precision
        $map.siblings('.description').find('span').html(
          (Math.acos(
            Math.sin(position.coords.latitude * Math.PI / 180)
              * Math.sin((position.coords.latitude) * Math.PI / 180)
            + Math.cos(position.coords.latitude * Math.PI / 180)
              * Math.cos((position.coords.latitude) * Math.PI / 180)
              * Math.cos(Math.pow(10, -1 * Drupal.settings.html5UserGeolocationPrecision) * Math.PI / 180)
          ) * 6371).toPrecision(3)
        );

        $busy.hide();
      }, function () { // getCurrentPosition error callback
        $('#edit-html5-user-geolocation-save').attr('checked', false).change();
      },
      {
        maximumAge: Infinity
      });
    }
    else { // Location not checked
      $('#html5-user-geolocation-map-wrapper').slideUp('fast');
    }
  }

  if (navigator.geolocation) {
    Drupal.behaviors.html5UserGeolocation = function (context) {
      $('#user-profile-form #edit-html5-user-geolocation-save:not(.html5-user-geolocation-processed)', context)
      .addClass('html5-user-geolocation-processed')
      .change(function () {
        getLocation();
      })
      .each(function () {
        $('#html5-user-geolocation-messages-wrapper .not-supported').hide();
        getLocation();
      });
    };
  }
  else { // HTML5 Geolocation not supported
    $('#edit-html5-user-geolocation-save').attr('disabled', true);
  }
}());
